package rpc

import (
	"context"
	"log"

	"github.com/valyala/fasthttp"

	"git.loafle.net/commons_go/rpc/gateway"
	"git.loafle.net/commons_go/rpc/protocol/json"
	"git.loafle.net/commons_go/server"
	cwf "git.loafle.net/commons_go/websocket_fasthttp"
	oogw "git.loafle.net/overflow/overflow_gateway_websocket"
	"git.loafle.net/overflow/overflow_gateway_websocket/rpc"
	"git.loafle.net/overflow/overflow_gateway_websocket/servlet"
)

func NewRPCGatewayServletHandler(rpcGH rpc.RPCGatewayHandler) RPCGatewayServletHandler {
	rpcSH := newRPCGatewayServerHandler()
	sh := &RPCGatewayServletHandlers{
		rpcSH: rpcSH,
		rpcGH: rpcGH,
	}

	return sh
}

type RPCGatewayServletHandler interface {
	servlet.ServletHandler
}

type RPCGatewayServletHandlers struct {
	servlet.ServletHandlers

	rpcSH RPCGatewayServerHandler
	rpcGH rpc.RPCGatewayHandler
}

func (sh *RPCGatewayServletHandlers) Init() error {
	if err := sh.ServletHandlers.Init(); nil != err {
		return err
	}

	if err := sh.rpcGH.Init(); nil != err {
		return err
	}

	return nil
}

func (sh *RPCGatewayServletHandlers) Handshake(ctx *fasthttp.RequestCtx) (id string, extensionsHeader *fasthttp.ResponseHeader) {
	return sh.rpcGH.Handshake(ctx)
}

// OnConnect invoked when client is connected
// If you override ths method, must call
func (sh *RPCGatewayServletHandlers) OnConnect(soc cwf.Socket) cwf.Socket {
	soc = sh.ServletHandlers.OnConnect(soc)

	return newSocket(soc, json.Name)
}

func (sh *RPCGatewayServletHandlers) Handle(soc cwf.Socket, stopChan <-chan struct{}, doneChan chan<- struct{}) {
	rpcSoc := soc.(Socket)
	codec, err := sh.rpcSH.GetCodec(rpcSoc.GetContentType())
	if nil != err {
		log.Printf("RPC Handle: %v", err)
		doneChan <- struct{}{}
		return
	}

	var socConn *cwf.SocketConn
	ctx := context.WithValue(context.Background(), oogw.ServletSocketKey, rpcSoc)
	// conn.SetPongHandler(func(string) error { c.conn.SetReadDeadline(time.Now().Add(pongWait)); return nil })

	for {
		if socConn, err = rpcSoc.WaitRequest(); nil != err {
			doneChan <- struct{}{}
			return
		}

		// // "git.loafle.net/commons_go/websocket_fasthttp/websocket"
		// switch socConn.MessageType {
		// case websocket.TextMessage:
		// case websocket.BinaryMessage:
		// }

		if err = gateway.Handle(ctx, sh.rpcSH, codec, socConn, socConn); nil != err {
			if server.IsClientDisconnect(err) {
				doneChan <- struct{}{}
				return
			}
			log.Printf("RPC: %v", err)
		}

		if err = socConn.Close(); nil != err {
			doneChan <- struct{}{}
			return
		}

		select {
		case <-stopChan:
			return
		default:
		}
	}

}

// OnDisconnect invoked when client is disconnected
// If you override ths method, must call
func (sh *RPCGatewayServletHandlers) OnDisconnect(soc cwf.Socket) {

	sh.ServletHandlers.OnDisconnect(soc)
}

// Destroy invoked when server is stopped
// If you override ths method, must call
func (sh *RPCGatewayServletHandlers) Destroy() {

	sh.ServletHandlers.Destroy()
}

func (sh *RPCGatewayServletHandlers) Validate() {
	sh.ServletHandlers.Validate()

	sh.rpcGH.Validate()
}
